@@ -82,7 +82,7 @@ auth = Auth(session, db, define_tables=False)
 auth.use_username = True
 auth.param.registration_requires_confirmation = settings.VERIFY_EMAIL
 auth.param.registration_requires_approval = settings.REQUIRES_APPROVAL
-auth.allowed_actions = ["all"]
+auth.allowed_actions = settings.ALLOWED_ACTIONS
 auth.login_expiration_time = 3600
 auth.password_complexity = {"entropy": 50}
 auth.block_previous_password_num = 3
