@@ -142,12 +142,13 @@ class Auth(Fixture):
         password_complexity={"entropy": 50},
         block_previous_password_num=None,
         allowed_actions=["all"],
-        use_appname_in_redirects=True,
+        use_appname_in_redirects=None,
     ):
 
         self.param = Param(
             registration_requires_confirmation=registration_requires_confirmation,
             registration_requires_approval=registration_requires_approval,
+            login_after_registration=False,
             login_expiration_time=login_expiration_time,  # seconds
             password_complexity=password_complexity,
             block_previous_password_num=block_previous_password_num,
@@ -176,6 +177,7 @@ class Auth(Fixture):
         self.use_username = use_username  # if False, uses email only
         self.use_phone_number = use_phone_number
         # The self._link variable is not thread safe (only intended for testing)
+        self.extra_auth_user_fields = extra_fields
         self._link = None
         self.extra_auth_user_fields = extra_fields
         if db and define_tables:
@@ -349,7 +351,9 @@ class Auth(Fixture):
             fields["username"] = fields.get("username", "").lower()
         fields["email"] = fields.get("email", "").lower()
         if self.param.registration_requires_confirmation:
-            token = str(uuid.uuid4()) + "/" + b16e(next)
+            token = str(uuid.uuid4())
+            if next:
+                token += "/" + b16e(next)
             fields["action_token"] = "pending-registration:%s" % token
             res = self.db.auth_user.validate_and_insert(**fields)
             if send and res.get("id"):
@@ -367,6 +371,8 @@ class Auth(Fixture):
         else:
             fields["action_token"] = ""
             res = self.db.auth_user.validate_and_insert(**fields)
+            if self.param.login_after_registration and not res.get("errors"):
+                self.store_user_in_session(res["id"])
         return res
 
     def login(self, email, password):
@@ -406,7 +412,9 @@ class Auth(Fixture):
             query = db.auth_user.email == value
         user = db(query).select().first()
         if user and not user.action_token == "account-blocked":
-            token = str(uuid.uuid4()) + "/" + b16e(next)
+            token = str(uuid.uuid4())
+            if next:
+                token += "/" + b16e(next)
             user.update_record(action_token="reset-password-request:" + token)
             if send:
                 self._link = link = URL(
@@ -629,6 +637,7 @@ class Auth(Fixture):
     def enable(self, route="auth", uses=(), env=None, spa=False):
         """enables Auth, aka generates login/logout/register/etc pages"""
         self.route = route = route.rstrip("/")
+        env = env or {}
         auth = self
 
         def allowed(name):
@@ -673,7 +682,7 @@ class Auth(Fixture):
                 @action(route + "/" + form_name, method=["GET", "POST"])
                 @action.uses(route + ".html")
                 @action.uses(auth, self.flash, *uses)
-                def _(form_factory=form_factory, path=form_name):
+                def _(form_factory=form_factory, path=form_name, env=env):
                     return dict(form=form_factory(), path=path, **env)
 
         for form_name in self.form_source.private_forms:
@@ -683,7 +692,7 @@ class Auth(Fixture):
                 @action(route + "/" + form_name, method=["GET", "POST"])
                 @action.uses(route + ".html")
                 @action.uses(auth.user, self.flash, *uses)
-                def _(auth=auth, form_factory=form_factory, path=form_name):
+                def _(auth=auth, form_factory=form_factory, path=form_name, env=env):
                     return dict(
                         form=form_factory(), path=path, user=auth.get_user(), **env
                     )
@@ -695,7 +704,7 @@ class Auth(Fixture):
                 @action(route + "/" + form_name)
                 @action.uses(route + ".html")
                 @action.uses(auth, self.flash, *uses)
-                def _(auth=auth, form_factory=form_factory, path=form_name):
+                def _(auth=auth, form_factory=form_factory, path=form_name, env=env):
                     return dict(
                         form=form_factory(), path=path, user=auth.get_user(), **env
                     )
@@ -874,6 +883,8 @@ class DefaultAuthForms:
             if not form.errors:
                 self._set_flash("user-registered")
                 self._postprocessing("register", form, user)
+                if self.auth.param.login_after_registration:
+                    redirect("login")
         form.param.sidecar.append(
             A("Sign In", _href="../auth/login", _class="info", _role="button")
         )
@@ -895,11 +906,7 @@ class DefaultAuthForms:
             fields[0].label = self.auth.db.auth_user.email.label
         fields[1].label = self.auth.db.auth_user.password.label
 
-        form = Form(
-            fields,
-            submit_value="Sign In",
-            formstyle=self.formstyle,
-        )
+        form = Form(fields, submit_value="Sign In", formstyle=self.formstyle,)
         user = None
         self.auth.next["login"] = request.query.get("next")
         if form.submitted:
@@ -917,17 +924,20 @@ class DefaultAuthForms:
             if self.auth.next["login"]:
                 url = url + "?next=" + self.auth.next["login"]
             top_buttons.append(A(plugin.label + " Login", _href=url, _role="button"))
-        form.param.sidecar.append(
-            A("Sign Up", _href="../auth/register", _class="info", _role="button")
-        )
-        form.param.sidecar.append(
-            A(
-                "Lost Password",
-                _href="../auth/request_reset_password",
-                _class="info",
-                _role="button",
+
+        if "register" in self.auth.param.allowed_actions:
+            form.param.sidecar.append(
+                A("Sign Up", _href="../auth/register", _class="info", _role="button")
+            )
+        if "request_reset_password" in self.auth.param.allowed_actions:
+            form.param.sidecar.append(
+                A(
+                    "Lost Password",
+                    _href="../auth/request_reset_password",
+                    _class="info",
+                    _role="button",
+                )
             )
-        )
         form.structure.insert(0, DIV(DIV(*top_buttons)))
         return form
 
