@@ -65,7 +65,7 @@ class FormStyleFactory:
     def produce(self, table, vars, errors, readonly, deletable, classes=None):
         self.classes.update(classes or {})
         form = FORM(_method="POST", _action=request.url, _enctype="multipart/form-data")
-        controls = dict(
+        controls = Param(
             labels=dict(),
             widgets=dict(),
             comments=dict(),
@@ -74,6 +74,8 @@ class FormStyleFactory:
             titles=dict(),
             errors=dict(),
             begin=XML(form.xml().split("</form>")[0]),
+            submit='',
+            delete='',
             end=XML("</form>"),
         )
         class_label = self.classes["label"]
@@ -112,7 +114,17 @@ class FormStyleFactory:
                 continue
             # if the form is readonly or this is an id type field, display it as readonly
             if readonly or not field.writable or field.type == "id":
-                control = DIV(field.represent and field.represent(value) or value or "")
+                if field.type == "boolean":
+                        control = INPUT(
+                        _type="checkbox",
+                        _id=input_id,
+                        _name=field.name,
+                        _value="ON",
+                        _disabled="",
+                        _checked=value,
+                        _title=title,)
+                else:
+                        control = DIV(field.represent and field.represent(value) or value or "")
             # if we have a widget for the field use it
             elif field.widget:
                 control = field.widget(table, value)
@@ -183,7 +195,7 @@ class FormStyleFactory:
                     control.append(download_div)
                 control.append(LABEL("Change: "))
                 control.append(INPUT(_type="file", _id=input_id, _name=field.name))
-            elif get_options(field.requires) is not None:
+            elif get_options(field.requires) is not None and field.writable==True:
                 multiple = field.type.startswith("list:")
                 value = list(map(str, value if isinstance(value, list) else [value]))
                 option_tags = [
@@ -405,7 +417,7 @@ class Form(object):
 
         # computed from input and not changed
         self.table = table
-        self.deletable = deletable and not readonly and self.record
+        self.deletable = self.record and deletable and not readonly
         self.dbio = dbio
         self.keep_values = True if keep_values or self.record else False
         self.form_name = form_name or table._tablename
@@ -451,7 +463,7 @@ class Form(object):
                                 and len(original_value) == 1
                             ):
                                 original_value = original_value[0]
-                            if field.type.startswith("list:"):
+                            if field.type.startswith("list:") and isinstance(original_value, str):
                                 original_value = json.loads(original_value or "[]")
                             (value, error) = field.validate(original_value, record_id)
                             if field.type == "password" and record_id and value is None:
