@@ -32,7 +32,7 @@ HEADER = TAG.header
 
 def title(text):
     """Turns text into a title"""
-    return text.replace("_", " ").title()
+    return str(text).replace("_", " ").title()
 
 
 def safe_int(text, default):
@@ -62,6 +62,7 @@ class GridClassStyle:
         "grid-sorter-icon-down": "grid-sort-icon-down fas fa-sort-down",
         "grid-th-action-button": "grid-col-action-button",
         "grid-td-action-button": "grid-col-action-button",
+        "grid-thead": "",
         "grid-tr": "",
         "grid-th": "",
         "grid-td": "",
@@ -152,6 +153,7 @@ class GridClassStyleBulma(GridClassStyle):
         "grid-sorter-icon-down": "grid-sort-icon-down fas fa-sort-down is-pulled-right",
         "grid-th-action-button": "grid-col-action-button is-narrow",
         "grid-td-action-button": "grid-col-action-button is-narrow",
+        "grid-thead": "",
         "grid-tr": "",
         "grid-th": "",
         "grid-td": "",
@@ -251,6 +253,12 @@ class Grid:
         )
         if value
         else "",
+        "list:string": lambda value: ', '.join(str(x) for x in value)
+        if value
+        else "",
+        "list:integer": lambda value: ', '.join(x for x in value)
+        if value
+        else "",
         "default": lambda value: str(value) if value is not None else ""
     }
 
@@ -548,6 +556,7 @@ class Grid:
         message=None,
         row_id=None,
         name="grid-button",
+        row=None,
         **attr,
     ):
         separator = "?"
@@ -571,6 +580,9 @@ class Grid:
         elif additional_styles:
             styles += join(additional_styles)
 
+        if callable(url):
+            url = url(row)
+
         link = A(
             I(_class="fa %s" % icon),
             _href=url,
@@ -695,7 +707,7 @@ class Grid:
                     col = A(heading, dw if "~" + key == sort_order else "", _href=href)
                 columns.append((key, col))
 
-        thead = THEAD()
+        thead = THEAD(_class=self.param.grid_class_style.classes.get("grid-thead", ""))
         for key, col in columns:
             col_class = "grid-col-%s" % key
             thead.append(
@@ -804,6 +816,7 @@ class Grid:
                                 additional_classes=btn.additional_classes,
                                 message=btn.message,
                                 row_id=row_id if btn.append_id else None,
+                                row=row
                             )
                         )
                 if self.param.details and self.param.details != "":
@@ -862,6 +875,7 @@ class Grid:
                                 additional_classes=btn.additional_classes,
                                 message=btn.message,
                                 row_id=row_id if btn.append_id else None,
+                                row=row
                             )
                         )
                 tr.append(td)
@@ -924,7 +938,7 @@ class Grid:
         #  build the search form if provided
         if self.param.search_form:
             grid_header.append(self.render_search_form())
-        else:
+        elif self.param.search_queries and len(self.param.search_queries) > 0:
             grid_header.append(self.render_default_form())
 
         html.append(grid_header)
